package com.dataguise.combinesmallfiles.avro;

/**
 * Created by shiyanghuang on 17/12/14.
 */

import com.dataguise.combinesmallfiles.text.WordOffset;
import org.apache.avro.file.DataFileReader;
import org.apache.avro.file.FileReader;
import org.apache.avro.file.SeekableInput;
import org.apache.avro.generic.GenericDatumReader;
import org.apache.avro.generic.GenericRecord;
import org.apache.avro.io.DatumReader;
import org.apache.avro.mapred.FsInput;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.mapreduce.InputSplit;
import org.apache.hadoop.mapreduce.RecordReader;
import org.apache.hadoop.mapreduce.TaskAttemptContext;
import org.apache.hadoop.mapreduce.lib.input.CombineFileSplit;

import java.io.IOException;

/**
 * A new AVRO record reader to use new MR API.
 *
 * @author john.zeng
 *
 */
public class AvroCombineRecordReader extends RecordReader<WordOffset, GenericRecord> {

    private long start;
    private long end;
    private boolean more = true;
    private WordOffset key = null;
    private GenericRecord value = null;
    protected Configuration conf;
    private Path path;
    private Integer curIndex = 0;
    private Integer prevIndex = -1;
    long startOffset;

    private FileReader<GenericRecord> in;
    private GenericRecord currentRow;
    private Integer rowNum = 0;  // key starts from 1

    private DatumReader<GenericRecord> reader;

    public AvroCombineRecordReader(CombineFileSplit split, TaskAttemptContext context, Integer index)
            throws IOException, InterruptedException {
        Configuration conf = context.getConfiguration();
        path = split.getPath(index);
        FileSystem fs = path.getFileSystem(conf);
        SeekableInput input = new FsInput(path, conf);
        reader = new GenericDatumReader<GenericRecord>();
        in = DataFileReader.openReader(input, reader);

        startOffset = split.getOffset(index);//offset of the chunk;
        this.end = startOffset + split.getLength(index);
        if (startOffset > in.tell()) {
            in.sync(startOffset);                  // sync to start
        }
        this.start = in.tell();
        more = start < end;
        // when we process a new file, we update prev file index and cur index
        curIndex = index;
    }

    @Override
    public void initialize(InputSplit inputSplit, TaskAttemptContext taskAttemptContext) throws IOException, InterruptedException {
    }

    @Override
    public boolean nextKeyValue() throws IOException, InterruptedException {

        if (in.hasNext()) {
            currentRow = in.next();
            rowNum++;
            return true;
        }
        else {
            currentRow = null;
            return false;
        }
    }

    @Override
    public WordOffset getCurrentKey() throws IOException, InterruptedException {
        if (key == null) {
            key = new WordOffset();
            key.fileName = path.getName();
        }
        key.offset = start;

        return key;
    }

    @Override
    public GenericRecord getCurrentValue() throws IOException, InterruptedException {
        if (currentRow != null) {
            return currentRow;
        }
        else return null;
    }

    @Override
    public float getProgress() throws IOException, InterruptedException {
        if (start == end) {
            return 0.0f;
        } else {
            return Math.min(1.0f, (start - startOffset) / (float)(end - startOffset));
        }
    }

    @Override
    public void close() throws IOException {
        in.close();
    }

}
