package com.dataguise.combinesmallfiles.avro;

/**
 * Created by shiyanghuang on 17/12/14.
 */

import org.apache.avro.file.DataFileReader;
import org.apache.avro.file.FileReader;
import org.apache.avro.file.SeekableInput;
import org.apache.avro.generic.GenericDatumReader;
import org.apache.avro.generic.GenericRecord;
import org.apache.avro.io.DatumReader;
import org.apache.avro.mapred.FsInput;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.mapreduce.InputSplit;
import org.apache.hadoop.mapreduce.RecordReader;
import org.apache.hadoop.mapreduce.TaskAttemptContext;
import org.apache.hadoop.mapreduce.lib.input.FileSplit;

import java.io.IOException;

/**
 * A new AVRO record reader to use new MR API.
 *
 * @author john.zeng
 *
 */
public class DgAvroRecordReader extends RecordReader<Integer, GenericRecord> {

    private FileReader<GenericRecord> fileReader;
    private GenericRecord currentRow;
    private Integer rowNum = 0;  // key starts from 1

    @Override
    public void initialize(InputSplit split, TaskAttemptContext context)
            throws IOException, InterruptedException {
        FileSplit fileSplit = (FileSplit) split;
        Configuration conf = context.getConfiguration();
        Path path = fileSplit.getPath();
        SeekableInput input = new FsInput(path, conf);
        DatumReader<GenericRecord> reader = new GenericDatumReader<GenericRecord>();
        fileReader = DataFileReader.openReader(input, reader);

    }

    @Override
    public boolean nextKeyValue() throws IOException, InterruptedException {
        if (fileReader.hasNext()) {
            currentRow = fileReader.next();
            rowNum++;
            return true;
        }
        else {
            currentRow = null;
            return false;
        }
    }

    @Override
    public Integer getCurrentKey() throws IOException, InterruptedException {
        if (currentRow != null)
            return rowNum;
        else return null;
    }

    @Override
    public GenericRecord getCurrentValue() throws IOException, InterruptedException {
        if (currentRow != null) {
            return currentRow;
        }
        else return null;
    }

    @Override
    public float getProgress() throws IOException, InterruptedException {
        // TODO Auto-generated method stub
        return 0;
    }

    @Override
    public void close() throws IOException {
        fileReader.close();
    }

}
