package com.dataguise.combinesmallfiles.orc;

import com.dataguise.combinesmallfiles.text.WordOffset;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.hive.ql.io.orc.*;
import org.apache.hadoop.mapreduce.InputSplit;
import org.apache.hadoop.mapreduce.RecordReader;
import org.apache.hadoop.mapreduce.TaskAttemptContext;
import org.apache.hadoop.mapreduce.lib.input.CombineFileSplit;

import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.List;

public class ORCCombineRecordReader extends RecordReader<WordOffset, OrcStruct> {
    private org.apache.hadoop.hive.ql.io.orc.RecordReader recordReader;
	private long start;
	private long end;
    private String fileName;
	private boolean more = true;
	private WordOffset key = null;
	private OrcStruct value = null;
	protected Configuration conf;
	private Path path;
	private Integer curIndex = 0;
	private Integer prevIndex = -1;
    private Object currentRow;
    private Integer rowNum = 0;
    private int numColumns;
    private float progress = 0.0F;

	public ORCCombineRecordReader(CombineFileSplit split, TaskAttemptContext context, Integer index) throws IOException, InterruptedException, IllegalAccessException, InstantiationException, NoSuchMethodException, InvocationTargetException {

        path = split.getPath(index);
		conf = context.getConfiguration();
        long startOffset = split.getOffset(index);//offset of the chunk;
	    this.end = startOffset + split.getLength(index);

        Reader file = OrcFile.createReader(path, OrcFile.readerOptions(conf));
        List types = file.getTypes();
        this.numColumns = types.size() == 0?0:((OrcProto.Type)types.get(0)).getSubtypesCount();


        Class<?> cls = null;
        try {
            cls = Class.forName("org.apache.hadoop.hive.ql.io.orc.OrcStruct");
            // Class<?> cls = Class.forName("org.apache.hadoop.hive.ql.io.orc.OrcNewInputFormat");
            Constructor[] cos = cls.getDeclaredConstructors();
            System.out.println(cos.length);
            Constructor constructor2 = cls.getDeclaredConstructor(new Class[]{int.class});
            //获取构造函数信息
            //通过带参数的构造函数创建ConstructorDemo类的实例
            constructor2.setAccessible(true);
            // Object obj2 = constructor2.newInstance(new Object[]{33});
            this.value = (OrcStruct) constructor2.newInstance(this.numColumns);
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }

        this.recordReader = OrcInputFormat.createReaderFromFile(OrcFile.createReader(path, OrcFile.readerOptions(conf)), conf, startOffset, split.getLength());
        this.fileName = path.toString();
		curIndex = index;
	}

	@Override
	public void initialize(InputSplit split, TaskAttemptContext context) throws IOException, InterruptedException {
	}

	@Override
	@SuppressWarnings("unchecked")
	public boolean nextKeyValue() throws IOException, InterruptedException {
        if(this.recordReader.hasNext()) {
            this.recordReader.next(this.value);
            this.progress = this.recordReader.getProgress();
            this.start = this.recordReader.getRowNumber();
            return true;
        } else {
            return false;
        }
    }

	@Override
	/**
	 * returned key would be original key + ? + file name path
	 * file name path updated once for each file
	 */
	public WordOffset getCurrentKey() throws IOException {
		String appender = "?";
        this.key = new WordOffset();
		//update on the first record of the file
		if (curIndex != prevIndex) {
			prevIndex = curIndex;
		}
        this.key.offset = this.start;
        this.key.index = this.curIndex;
        this.key.fileName = this.fileName;
		return this.key;
	}


	@Override
	public OrcStruct getCurrentValue() {
		return value;
	}

	public float getProgress() throws IOException {
        return recordReader.getProgress();
    }

	public synchronized void close() throws IOException {
        this.recordReader.close();
	}

}