package com.dataguise.combinesmallfiles.rc;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.hive.ql.io.RCFile.Reader;
import org.apache.hadoop.hive.serde2.columnar.BytesRefArrayWritable;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.mapreduce.InputSplit;
import org.apache.hadoop.mapreduce.RecordReader;
import org.apache.hadoop.mapreduce.TaskAttemptContext;
import org.apache.hadoop.mapreduce.lib.input.CombineFileSplit;
import org.apache.hadoop.mapreduce.lib.input.FileSplit;

import java.io.IOException;

/**
 * RCFileRecordReader.
 * @see RCCombineFileInputFormat.java
 *
 */
public class RCCombineFileRecordReader extends RecordReader {
	private Reader in;
	private long start;
	private long end;
	private boolean more = true;
	private LongWritable key = null;
	private BytesRefArrayWritable value = null;
	protected Configuration conf;
    private LongWritable curIndex = null;

    public RCCombineFileRecordReader(CombineFileSplit split, TaskAttemptContext context, Integer index) throws IOException {

        conf = context.getConfiguration();
        Path path = split.getPath(index);
        FileSystem fs = path.getFileSystem(conf);
        this.in = new Reader(fs, path, conf);
        this.end = split.getStartOffsets()[index] + split.getLength();

        if (split.getStartOffsets()[index] > in.getPosition()) {
            in.sync(split.getStartOffsets()[index]); // sync to start
        }
        this.start = in.getPosition();
        more = start < end;
        curIndex = new LongWritable(index);
    }

	/**
	  * Return the progress within the input split.
	  * 
	  * @return 0.0 to 1.0 of the input byte range
	  */
	public float getProgress() throws IOException {
		if (end == start) {
			return 0.0f;
		} else {
			return Math.min(1.0f, (in.getPosition() - start)
					/ (float) (end - start));
		}
	}
	 
	public void close() throws IOException {
		in.close();
	}
	 
	@Override
	public LongWritable getCurrentKey() throws IOException,
	   InterruptedException {
		return curIndex;
	}
	 
	@Override
	public BytesRefArrayWritable getCurrentValue() throws IOException,
	   InterruptedException {
		return value;
	}
	 
	@Override
	public void initialize(InputSplit split, TaskAttemptContext context)
			throws IOException, InterruptedException {
	}
	 
	@Override
	public boolean nextKeyValue() throws IOException, InterruptedException {
		if (!more) {
			return false;
		}
		
		if (key == null) {
			key = new LongWritable();
		}
		
		if (value == null) {
			value = new BytesRefArrayWritable();
		}
	  
		more = in.next(key);
		if (!more) {
			return false;
		}
	  
		long lastSeenSyncPos = in.lastSeenSyncPos();
		if (lastSeenSyncPos >= end) {
			more = false;
			return more;
		}
	  
		in.getCurrentRow(value);
		return more;
	}
}