package com.dataguise.combinesmallfiles.rc;

/**
 * Created by shiyanghuang on 17/12/14.
 */

import com.google.common.io.ByteStreams;
import org.apache.hadoop.hive.serde2.columnar.BytesRefArrayWritable;
import org.apache.hadoop.hive.serde2.columnar.BytesRefWritable;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.WritableComparable;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.log4j.Logger;

import java.io.DataInput;
import java.io.IOException;
import java.util.Arrays;
import java.util.StringTokenizer;

/**
 * This Mapper is similar to the one in .
 */
public class RCMap extends
        Mapper<Object, Object, Text, IntWritable> {
    private final static IntWritable one = new IntWritable(1);
    private Text word = new Text();
    Logger logger = Logger.getLogger(RCMap.class);

    public void setup(Context context) {
        logger.info(context.getInputSplit());
    }
    public void map(Object key, Object value, Context context)
            throws IOException, InterruptedException {
        logger.info("KEY: " + key);
        String[] columns = convertRCtoStringArray((BytesRefArrayWritable)value, (WritableComparable<Long>)key, logger);
        for (String s : columns) {
            logger.info(s);
        }

        String line = value.toString();
        StringTokenizer itr = new StringTokenizer(line);
        while (itr.hasMoreTokens()) {
            word.set(itr.nextToken());
            context.write(word, one);
        }
    }

    public static String[] convertRCtoStringArray(BytesRefArrayWritable value, WritableComparable<Long> key,
                                                  Logger logger) throws IOException {
        int numberOfColumns = value.size();
        String[] columns = new String[numberOfColumns];

        for (int i=0; i<numberOfColumns; i++) {
            BytesRefWritable columnValue = value.get(i);
            logger.debug("element ["+key+","+i+"] length is "+columnValue.getLength());
            logger.debug("element ["+key+","+i+"] start is "+columnValue.getStart());

            //
            // convert column value from byte array to string format
            // 'getData' returns same for all columns in a row-group.  but each column has different 'start' and 'length'.
            //
            byte[] currentColumnBytes = Arrays.copyOfRange(columnValue.getData(),
                    columnValue.getStart(), columnValue.getStart() + columnValue.getLength());

            DataInput dataInput = ByteStreams.newDataInput(currentColumnBytes);
            String currentColumn = dataInput.readLine();
            columns[i] = currentColumn;
        }
        return columns;
    }
}