package com.dataguise.combinesmallfiles.sequence;

import com.dataguise.util.Config;
import com.dataguise.util.DC;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
import org.apache.hadoop.mapreduce.lib.reduce.IntSumReducer;
import org.apache.hadoop.security.UserGroupInformation;
import org.apache.parquet.hadoop.ParquetInputFormat;
import org.apache.parquet.hadoop.example.GroupReadSupport;

import java.security.PrivilegedExceptionAction;

/**
 * MultiFileWordCount is an example to demonstrate the usage of
 * MultiFileInputFormat. This examples counts the occurrences of
 * words in the text files under the given input directory.
 */
public class MultiSequenceWordCount {

    public static void runMultiFiles(String confPath, final String input, final String output, final String jarPath, String user) {

        Configuration conf = new Configuration();

        Config.setConf(confPath, conf);

        UserGroupInformation.setConfiguration(conf);
        UserGroupInformation ugi = UserGroupInformation.createRemoteUser(user);

        final Configuration fconf = conf;
        try {

            ugi.doAs(new PrivilegedExceptionAction() {

                public Void run() throws Exception {

                    DC.addJarToDistributedCache(jarPath, fconf);

                    Job job = new Job(fconf);
                    job.setJobName("MultiSequenceWordCount");
                    job.setJarByClass(MultiSequenceWordCount.class);

                    //set the InputFormat of the job to our InputFormat
                    job.setInputFormatClass(SequenceCombineInputFormat.class);
                    // job.setInputFormatClass(SequenceFileInputFormat.class);

                    // the keys are words (strings)
                    job.setOutputKeyClass(Text.class);
                    // the values are counts (ints)
                    job.setOutputValueClass(IntWritable.class);

                    //use the defined mapper
                    job.setMapperClass(SequencetMap.class);
                    //use the WordCount Reducer
                    job.setCombinerClass(IntSumReducer.class);
                    job.setReducerClass(IntSumReducer.class);

                    String rand = Math.random() + "";
                    String outputNew = output + rand;

                    FileInputFormat.addInputPaths(job, input);
                    FileOutputFormat.setOutputPath(job, new Path(outputNew));
                    ParquetInputFormat.setReadSupportClass(job, GroupReadSupport.class);

                    job.waitForCompletion(true);
                    return null;
                }
            });

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void main(String[] args) throws Exception {
        String confPath = "/Users/shiyanghuang/Documents/Dataguise/hdp_conf/conf";
        String input = "/AllFiles/Sequence/BuiltIn/Simple/SequenceFile_Simple_Uncompressed";
        String output = "/wordcount/output_sequence";
        String jarPath = "/Users/shiyanghuang/Documents/Dataguise/hadoop/hadoopUtilTools/out/artifacts/hadoopUtilTools_jar/hadoopUtilTools.jar";
        String user = "hdfs";
        runMultiFiles(confPath, input, output, jarPath, user);
    }
}
