package com.dataguise.combinesmallfiles.sequence;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.DefaultStringifier;
import org.apache.hadoop.io.SequenceFile;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.InputSplit;
import org.apache.hadoop.mapreduce.RecordReader;
import org.apache.hadoop.mapreduce.TaskAttemptContext;
import org.apache.hadoop.mapreduce.lib.input.CombineFileSplit;

import java.io.IOException;

public class SequenceCombineRecordReader<K, V> extends RecordReader<Text, V> {
	private SequenceFile.Reader in;
	private long start;
	private long end;
	private boolean more = true;
	private K key = null;
	private V value = null;
	protected Configuration conf; 
	private Path path;
	private Integer curIndex = 0;
	private Integer prevIndex = -1;
	
	public SequenceCombineRecordReader(CombineFileSplit split, TaskAttemptContext context, Integer index) throws IOException {
		path = split.getPath(index);
		conf = context.getConfiguration();  
		FileSystem fs = path.getFileSystem(conf);
		this.in = new SequenceFile.Reader(fs, path, conf);
		long startOffset = split.getOffset(index);//offset of the chunk;
	    this.end = startOffset + split.getLength(index);
		if (startOffset > in.getPosition()) {
			in.sync(startOffset);                  // sync to start
		}
		this.start = in.getPosition();
		more = start < end;
		// when we process a new file, we update prev file index and cur index
		curIndex = index;
	}

	@Override
	public void initialize(InputSplit split, TaskAttemptContext context) throws IOException, InterruptedException {
	}
	
	@Override
	@SuppressWarnings("unchecked")
	public boolean nextKeyValue() throws IOException, InterruptedException {
		if (!more) {
			return false;
		}
		
		long pos = in.getPosition();
		key = (K) in.next(key);
		if (key == null || (pos >= end && in.syncSeen())) {
			more = false;
			key = null;
			value = null;
		} else {
			value = (V) in.getCurrentValue(value);
		}
		
		return more;
	}
	
	@Override
	/**
	 * returned key would be original key + ? + file name path
	 * file name path updated once for each file
	 */
	public Text getCurrentKey() throws IOException {
		String appender = "?";
		
		//update on the first record of the file
		if (curIndex != prevIndex) {
			prevIndex = curIndex;
			appender = "?" + path.toString();
		}
		DefaultStringifier keyStringifier = null;
		keyStringifier = new DefaultStringifier(conf, key.getClass());
		return new Text(path.getName());
	}
	
	
	@Override
	public V getCurrentValue() {
		return value;
	}
	
	public float getProgress() throws IOException {
		if (end == start) {
			return 0.0f;
		} else {
			return Math.min(1.0f, (in.getPosition() - start) / (float)(end - start));
		}
	}
	
	public synchronized void close() throws IOException { 
		in.close(); 
	}
	
}