package com.dataguise.combinesmallfiles.text;

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


import com.dataguise.util.Config;
import com.dataguise.util.DC;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
import org.apache.hadoop.mapreduce.lib.reduce.IntSumReducer;
import org.apache.hadoop.security.UserGroupInformation;

import java.io.IOException;
import java.security.PrivilegedExceptionAction;


/**
 * MultiFileWordCount is an example to demonstrate the usage of
 * MultiFileInputFormat. This examples counts the occurrences of
 * words in the text files under the given input directory.
 */
public class MultiFileWordCount {

    public static void runMultiFiles(String confPath, final String input, final String output, final String jarPath, String user) throws IOException {

        Configuration conf = new Configuration();

        Config.setConf(confPath, conf);

        UserGroupInformation.setConfiguration(conf);
        UserGroupInformation ugi = UserGroupInformation.createRemoteUser(user);

        final Configuration fconf = conf;
        try {

            ugi.doAs(new PrivilegedExceptionAction() {

                public Void run() throws Exception {
                    DC.addJarToDistributedCache(jarPath, fconf);

//                    FileSystem fs = FileSystem.get(fconf);
//                    FileStatus[] fileList = fs.listStatus(new Path("/testfile10x10MB"));
//                    String newInput = "";
//                    for (FileStatus s : fileList) {
//                        newInput += s.getPath() + ",";
//                    }
//                    newInput = newInput.substring(0, newInput.length() - 1);

//                    System.out.println(newInput);

                    Job job = new Job(fconf);
                    job.setJobName("MultiFileWordCount");
                    job.setJarByClass(MultiFileWordCount.class);

                    //set the InputFormat of the job to our InputFormat
                    job.setInputFormatClass(MyInputFormat.class);

                    // the keys are words (strings)
                    job.setOutputKeyClass(Text.class);
                    // the values are counts (ints)
                    job.setOutputValueClass(IntWritable.class);

                    //use the defined mapper
                    job.setMapperClass(MapClass.class);
                    //use the WordCount Reducer
                    job.setCombinerClass(IntSumReducer.class);
                    job.setReducerClass(IntSumReducer.class);

                    String rand = Math.random() + "";
                    String outputNew = output + rand;
                    System.out.println(outputNew);
                    FileInputFormat.addInputPaths(job, input);
                    FileOutputFormat.setOutputPath(job, new Path(outputNew));

                    job.waitForCompletion(true);
                    return null;
                }
            });

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void main(String[] args) throws Exception {
        String confPath = "/Users/shiyanghuang/Documents/Dataguise/hdp_conf/conf";
        String input = "/wordcount/input";
        input = "/bug16153/grt_masking_file";
        String output = "/wordcount/output";
        String jarPath = "/Users/shiyanghuang/Documents/Dataguise/hadoop/hadoopUtilTools/out/artifacts/hadoopUtilTools_jar/hadoopUtilTools.jar";
        String user = "hdfs";
        runMultiFiles(confPath, input, output, jarPath, user);
    }
}
