package com.dataguise.kerberos;

import com.dataguise.mapreduce.mapreduceMap;
import com.dataguise.mapreduce.mapreduceReduce;
import com.dataguise.util.Config;
import com.dataguise.util.DC;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.input.TextInputFormat;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
import org.apache.hadoop.mapreduce.lib.output.TextOutputFormat;
import org.apache.hadoop.security.UserGroupInformation;

import java.io.IOException;
import java.security.PrivilegedExceptionAction;

/**
 * Created by shiyanghuang on 18/2/16.
 */
public class submitKerberosJob {
    public static void main(String[] args) {
        String confPath = "/Users/shiyanghuang/Documents/Dataguise/hdp_conf/conf_kbr";
        String input = "/wordcount/input";
        String output = "/wordcount/output";
        String jarPath = "/Users/shiyanghuang/Documents/Dataguise/hadoop/hadoopUtilTools/out/artifacts/hadoopUtilTools_jar/hadoopUtilTools.jar";
        String user = "hdfs-hdp26@HDP26.DG";
        submitJob(confPath, input, output, jarPath, user);
    }

    public static void submitJob(String confPath, final String input, final String output, final String jarPath, String user) {
        Configuration conf = new Configuration();
        Config.setConf(confPath, conf);

        UserGroupInformation ugi = null;
        conf.set("hdp.version", "2.6.3.0-235");
        try {
            UserGroupInformation.setConfiguration(conf);
            ugi = UserGroupInformation.loginUserFromKeytabAndReturnUGI(user, confPath + "/hdfs.keytab");
            UserGroupInformation.getUGIFromTicketCache(confPath, user);
            // ugi = UserGroupInformation.createRemoteUser(user);
            // kinit(user, confPath + "/hdfs.keytab");
        } catch (IOException e) {
            System.out.println("Cannot get keytab" + e);
        }
        final Configuration fconf = conf;
        try {

            ugi.doAs(new PrivilegedExceptionAction() {

                public Void run() throws Exception {

                    Job job = null;
                    try {
                        DC.addJarToDistributedCache(jarPath, fconf);

                        job = Job.getInstance(fconf, "KBRWordcount");

                        job.setOutputKeyClass(Text.class);
                        job.setOutputValueClass(IntWritable.class);

                        job.setMapperClass(mapreduceMap.class);
                        job.setReducerClass(mapreduceReduce.class);

                        job.setInputFormatClass(TextInputFormat.class);
                        job.setOutputFormatClass(TextOutputFormat.class);

                        job.setNumReduceTasks(1);
                        job.setJarByClass(submitKerberosJob.class);

                        String rand = Math.random() + "";
                        String outputNew = output + rand;

                        FileInputFormat.addInputPath(job, new Path(input));
                        FileOutputFormat.setOutputPath(job, new Path(outputNew));

                        job.waitForCompletion(true);

                    } catch (IOException e) {
                        e.printStackTrace();
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    } catch (ClassNotFoundException e) {
                        e.printStackTrace();
                    }
                    return null;
                }
            });

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private static void kinit(String user, String path) {
        String command = "kinit -k -t " + path + " " + user;
        Process process = null;
        try {
            process = Runtime.getRuntime().exec(command);
            process.waitFor();
            process.destroy();
        } catch (Exception e) {
            System.out.println(e);
        }
    }
}
