package com.dataguise.mapred;
/*
* Created by shiyanghuang on 16/6/23.
        */

import com.dataguise.util.Config;
import com.dataguise.util.DC;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.*;
import org.apache.hadoop.security.UserGroupInformation;

import java.security.PrivilegedExceptionAction;

public class submitJobClientTest {

    public static void main(final String[] args) throws Exception {
        String confPath = "/Users/shiyanghuang/Documents/Dataguise/hdp_conf/conf";
        String input = "/wordcount/input";
        String output = "/wordcount/output";
        String jarPath = "/Users/shiyanghuang/Documents/Dataguise/hadoop/hadoopUtilTools/out/artifacts/hadoopUtilTools_jar/hadoopUtilTools.jar";
        String user = "hdfs";
        submitJobClient(confPath, input, output, jarPath, user);
    }

    public static void submitJobClient(String confPath, final String input, final String output, final String jarPath, String user) {

        Configuration conf = new Configuration();

        Config.setConf(confPath, conf);

        UserGroupInformation.setConfiguration(conf);
        UserGroupInformation ugi = UserGroupInformation.createRemoteUser(user);

        final Configuration fconf = conf;
        try {

            ugi.doAs(new PrivilegedExceptionAction() {

                public Void run() throws Exception {
                    DC.addJarToDistributedCache(jarPath, fconf);

                    JobConf jobConf = new JobConf(fconf);
                    jobConf.setJobName("MapredWordCount");

                    jobConf.setOutputKeyClass(Text.class);
                    jobConf.setOutputValueClass(IntWritable.class);

                    jobConf.setMapperClass(mapredMap.class);
                    jobConf.setReducerClass(mapredReduce.class);

                    jobConf.setInputFormat(TextInputFormat.class);
                    jobConf.setOutputFormat(TextOutputFormat.class);

                    jobConf.setNumReduceTasks(1);
                    jobConf.setJarByClass(submitJobClientTest.class);


                    String rand = Math.random() + "";
                    String outputNew = output + rand;

                    FileInputFormat.addInputPath(jobConf, new Path(input));
                    FileOutputFormat.setOutputPath(jobConf, new Path(outputNew));

                    JobClient jobClient = new JobClient(fconf);

                    RunningJob runningJob = jobClient.submitJob(jobConf);

                    while (!runningJob.isComplete()) {
                        System.out.println("Job status: " + runningJob.getJobState());
                        Thread.sleep(5000);
                    }

                    System.out.println("Job Complete");

                    return null;
                }
            });

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}